package com.diagbot.repository;

import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.LisBillNeoDTO;
import com.diagbot.dto.NeoEntityDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.entity.node.*;
import com.diagbot.util.NeoUtil;

import javax.xml.soap.Node;
import java.math.BigDecimal;
import java.util.*;

public class LisPackNode {

	public LisBillNeoDTO LisPacktoLISDTO(LISPack lispack) {

		LisBillNeoDTO lisBillNeoDTO = new LisBillNeoDTO();
		NodeInfo nodeInfo;

		Gender gender = lispack.getGender();
		if (null!=gender) {
			nodeInfo = new NodeInfo();
			nodeInfo.setName(gender.getName());
			nodeInfo.setTypeval("性别");
			lisBillNeoDTO.setGender(NeoUtil.updateNodeInfo(nodeInfo));
		}


		Age age = lispack.getAge();
		if (null!=age) {
			NodeNeoDTO ageNeoDTO = new NodeNeoDTO();
			ageNeoDTO.setMin(BigDecimal.valueOf(age.getMinval()));
			ageNeoDTO.setMax(BigDecimal.valueOf(age.getMaxval()));
			ageNeoDTO.setName("年龄");
			lisBillNeoDTO.setAgeNeoDTO(ageNeoDTO);
		}

		Set<ClinicalFinding> clinicalfindings = lispack.getClinicalfindings();
		for (ClinicalFinding clinicalfinding : clinicalfindings) {
			nodeInfo = new NodeInfo();
			nodeInfo.setName(clinicalfinding.getName());
			nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(clinicalfinding.getMinval()));
			nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(clinicalfinding.getMaxval()));
			nodeInfo.setUnit(clinicalfinding.getUnit());
			nodeInfo.setRange(clinicalfinding.getRange());
			lisBillNeoDTO.getClinicfindings().add(NeoUtil.updateNodeInfo(nodeInfo));
		}

		Set<Medicine> conflict_medicines = lispack.getConflict_medicines();
		for (Medicine conflict_medicine : conflict_medicines) {
			nodeInfo = new NodeInfo();
			nodeInfo.setName(conflict_medicine.getName());
			lisBillNeoDTO.getOralmeds().add(NeoUtil.updateNodeInfo(nodeInfo));
		}

		Set<AllergicMed> allergicmeds = lispack.getAllergicmeds();
		for (AllergicMed allergicmed : allergicmeds) {
			nodeInfo = new NodeInfo();
			nodeInfo.setName(allergicmed.getName());

			lisBillNeoDTO.getAllergicmeds().add(NeoUtil.updateNodeInfo(nodeInfo));
		}

		Set<Operation> operations = lispack.getOperations();
		for (Operation operation : operations) {
			nodeInfo = new NodeInfo();
			nodeInfo.setName(operation.getName());

			lisBillNeoDTO.getOperations().add(NeoUtil.updateNodeInfo(nodeInfo));
		}

		Set<Procedure> procedures = lispack.getProcedures();
		for (Procedure procedure : procedures) {
			nodeInfo = new NodeInfo();
			nodeInfo.setName(procedure.getName());
			lisBillNeoDTO.getProcedures().add(NeoUtil.updateNodeInfo(nodeInfo));
		}

		Set<PACS> pacss = lispack.getPacs();
		for (PACS pacs : pacss) {
			nodeInfo = new NodeInfo();
			nodeInfo.setName(pacs.getName());
			lisBillNeoDTO.getPacs().add(NeoUtil.updateNodeInfo(nodeInfo));
		}

		Set<Disease> diseases = lispack.getDiseases();
		for (Disease disease : diseases) {
			nodeInfo = new NodeInfo();
			nodeInfo.setName(disease.getName());
			lisBillNeoDTO.getDisease().add(NeoUtil.updateNodeInfo(nodeInfo));
		}

		Set<Group> groups = lispack.getGroups();
		for (Group group : groups) {
			nodeInfo = new NodeInfo();
			nodeInfo.setName(group.getName());
			lisBillNeoDTO.getGroup().add(NeoUtil.updateNodeInfo(nodeInfo));
		}


		return lisBillNeoDTO;
	}


	/**
	 * 处理化验套餐开单合理性
	 */
	public List<BillNeoDTO> getLisBill(Map<String, String> lispackterm, LisPackRepository lisPackRepository) {
		List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
		BillNeoDTO billNeoDTO;

		LisBillNeoDTO lisBillNeoDTO;

		List<LISPack> lispacks;
		String lispackname;

		for (String term : lispackterm.keySet()) {
			billNeoDTO = new BillNeoDTO();
			lispackname = lispackterm.get(term);
			billNeoDTO.setName(term);
			billNeoDTO.setStandname(lispackname);

			lisBillNeoDTO = new LisBillNeoDTO();

			lispacks = lisPackRepository.findByNameIs(lispackname);

			if (lispacks != null && lispacks.size()>0) {
				lisBillNeoDTO = LisPacktoLISDTO(lispacks.get(0));
			}

			billNeoDTO.setLisBillNeoDTO(lisBillNeoDTO);

			billNeoDTOs.add(billNeoDTO);
		}

		return billNeoDTOs;
	}



	/**
	 * 获取化验套餐名称列表
	 */
	public List<String> getLISPackNameList(String name, LisPackRepository lisPackRepository) {
		String nodename;

		List<String> lispacklist = new ArrayList<>();

		List<LISPack> lisPacks = lisPackRepository.findByNameLike("*"+name+"*");

		for (LISPack lisPack : lisPacks) {
			nodename = lisPack.getName();
			if (!lispacklist.contains(nodename)) {
				lispacklist.add(nodename);
			}
		}


		return lispacklist;
	}



	/**
	 * 更新实体属性信息
	 */
	public NeoEntityDTO updateEntity(String name, Map<String, String> props, List<String> keys, LisPackRepository lisPackRepository) {
		NeoEntityDTO neoEntityDTO = new NeoEntityDTO();
		neoEntityDTO.setName(name);

		try {
			List<LISPack> lispackres = lisPackRepository.findByNameIs(name);

			if (lispackres!= null && lispackres.size()>0) {
				LISPack lispack = lispackres.get(0);

				for (String key : props.keySet()) {
					switch (key) {
						case "最大值":
							lispack.setMaxval(Double.parseDouble(props.get(key)));
							break;
						case "最小值":
							lispack.setMinval(Double.parseDouble(props.get(key)));
							break;
						case "单位":
							lispack.setUnit(props.get(key));
							break;
					}
				}

				lispack = lisPackRepository.save(lispack);

				neoEntityDTO = fillNeoDTO(lispack, keys);
			}
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}

		return neoEntityDTO;
	}


	private NeoEntityDTO fillNeoDTO(LISPack lisPack, List<String> keys) {
		NeoEntityDTO neoEntityDTO = new NeoEntityDTO();
		Map<String, String> props = new HashMap<>();
		String val = "";

		neoEntityDTO.setName(lisPack.getName());
		for (String key:keys) {
			switch(key) {
				case "最大值":
					val = String.format("%.2f",lisPack.getMaxval());
					break;
				case "最小值":
					val = String.format("%.2f", lisPack.getMinval());
					break;
				case "单位":
					val = lisPack.getUnit();
					break;
			}
			props.put(key, val);
		}
		neoEntityDTO.setProperty(props);

		return neoEntityDTO;
	}

}

