package com.diagbot.repository;

import com.diagbot.dto.*;
import com.diagbot.entity.node.*;
import com.diagbot.util.NeoUtil;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class MedicineNode {


    public DrugBillNeoDTO MedicinetoDrugDTO(Medicine medicine) {
        DrugBillNeoDTO drugBillNeoDTO = new DrugBillNeoDTO();
        NodeInfo nodeInfo = new NodeInfo();

        Gender gender = medicine.getGender();
        if (gender!=null) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(gender.getName());
            nodeInfo.setTypeval("性别");
            drugBillNeoDTO.setGender(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Age age = medicine.getAge();
        if (age!=null) {
            NodeNeoDTO ageNeoDTO = new NodeNeoDTO();
            ageNeoDTO.setMin(BigDecimal.valueOf(age.getMinval()));
            ageNeoDTO.setMax(BigDecimal.valueOf(age.getMaxval()));
            drugBillNeoDTO.setAgeNeoDTO(ageNeoDTO);
        }


        Set<ClinicalFinding> clinicalfindings = medicine.getClinicalfindings();
        for (ClinicalFinding clinicalfinding : clinicalfindings) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(clinicalfinding.getName());
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(clinicalfinding.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(clinicalfinding.getMaxval()));
            nodeInfo.setUnit(clinicalfinding.getUnit());
            nodeInfo.setRange(clinicalfinding.getRange());
            drugBillNeoDTO.getClinicfindings().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<Vital> vitals = medicine.getVitals();
        for (Vital vital : vitals) {
            nodeInfo = new NodeInfo();
            String name = vital.getName();
            if (name.contains("禁忌")) {
                name = name.substring(name.indexOf("禁忌")+2);
            }
            nodeInfo.setName(name);
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(vital.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(vital.getMaxval()));
            nodeInfo.setUnit(vital.getUnit());
            nodeInfo.setRange(vital.getRange());
            drugBillNeoDTO.getVitals().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<LIS> lisset = medicine.getLisset();
        for (LIS lis : lisset) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(lis.getName());
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(lis.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(lis.getMaxval()));
            nodeInfo.setUnit(lis.getUnit());
            nodeInfo.setRange(lis.getRange());
            drugBillNeoDTO.getLis().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<PACS> pacsset = medicine.getPacs();
        for (PACS pacs : pacsset) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(pacs.getName());
            drugBillNeoDTO.getPacs().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<AllergicMed> allergicmeds = medicine.getAllergicmeds();
        for (AllergicMed allergicmed: allergicmeds) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(allergicmed.getName());
            drugBillNeoDTO.getAllergicmeds().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<Operation> operations = medicine.getOperations();
        for (Operation operation : operations) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(operation.getName());
            drugBillNeoDTO.getOperations().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<Procedure> procedures = medicine.getProcedures();
        for (Procedure procedure : procedures) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(procedure.getName());
            drugBillNeoDTO.getProcedures().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<Disease> diseases = medicine.getDiseases();
        for (Disease disease : diseases) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(disease.getName());
            drugBillNeoDTO.getDisease().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<Group> groups = medicine.getGroups();
        for (Group group : groups) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(group.getName());
            drugBillNeoDTO.getGroup().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<ConflictMed> conflict_medicines = medicine.getConflictmeds();
        for (ConflictMed conflict_medicine : conflict_medicines) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(conflict_medicine.getName());
            drugBillNeoDTO.getConflictmeds().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        return drugBillNeoDTO;
    }



    /**
     * 处理处方开单合理性
     */
    public List<BillNeoDTO> getDrugBill(Map<String, String> medsterm, MedicineRepository medicineRepository) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        DrugBillNeoDTO drugBillNeoDTO;

        List<Medicine> medicine;
        String medname;

        for (String term : medsterm.keySet()) {
            billNeoDTO = new BillNeoDTO();
            medname = medsterm.get(term);
            billNeoDTO.setName(term);
            billNeoDTO.setStandname(medname);

            drugBillNeoDTO = new DrugBillNeoDTO();

            medicine = medicineRepository.findByNameIs(medname);

            if (medicine != null && medicine.size()>0) {
                drugBillNeoDTO = MedicinetoDrugDTO(medicine.get(0));

                billNeoDTO.setDrugBillNeoDTO(drugBillNeoDTO);

                billNeoDTOs.add(billNeoDTO);
            }
        }


        return billNeoDTOs;
    }



    /**
     * 获取药品商品名称列表
     */
    public List<DrugInfoDTO> getMedicineList(String name, MedicineRepository medicineRepository) {
        String nodename;

        List<DrugInfoDTO> medslist = new ArrayList<>();
        DrugInfoDTO drugInfoDTO;

        List<Medicine> medicines = medicineRepository.findByNameLike("*"+name+"*");

        for (Medicine med : medicines) {
            drugInfoDTO = new DrugInfoDTO();
            nodename = med.getName();

            drugInfoDTO.setName(nodename);
            medslist.add(drugInfoDTO);
        }


        return medslist;
    }



    /**
     * 获取高危药品数据
     */
    public List<HighRiskNeoDTO> getHighRiskDrug(Map<String, String> drugsterm, MedicineRepository medicineRepository) {
        List<HighRiskNeoDTO> highRiskNeoDTOS = new ArrayList<>();
        HighRiskNeoDTO highRiskNeoDTO;
        String drugname;
        List<Medicine> medicine;
        Medicine med;

        for (String term : drugsterm.keySet()) {
            highRiskNeoDTO = new HighRiskNeoDTO();

            drugname = drugsterm.get(term);
            highRiskNeoDTO.setName(term);
            highRiskNeoDTO.setStandname(drugname);

            medicine = medicineRepository.findByNameIs(drugname);

            if (null != medicine && medicine.size()>0) {
                med = medicine.get(0);

                if (null!=med.getRisklevel() && med.getRisklevel().trim().length()>0) {
                    highRiskNeoDTO.setIsHighrisk(true);
                    highRiskNeoDTO.setType("药品");
                    highRiskNeoDTOS.add(highRiskNeoDTO);
                }
            }
        }

        return highRiskNeoDTOS;
    }
}

