package com.diagbot.repository;


import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.HighRiskNeoDTO;
import com.diagbot.dto.OperationBillNeoDTO;
import com.diagbot.dto.OperationInfoDTO;
import com.diagbot.entity.node.*;
import com.diagbot.util.NeoUtil;

import javax.xml.soap.Node;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class OperationNode {


    public OperationBillNeoDTO OperationtoOperationDTO(Operation operation) {
        OperationBillNeoDTO opBillNeoDTO = new OperationBillNeoDTO();
        NodeInfo nodeInfo;

        Gender gender = operation.getGender();
        if (null!=gender) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(gender.getName());
            opBillNeoDTO.setGender(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<ClinicalFinding> clinicalfindings = operation.getClinicalFindings();
        for (ClinicalFinding clinicalfinding : clinicalfindings) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(clinicalfinding.getName());
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(clinicalfinding.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(clinicalfinding.getMaxval()));
            nodeInfo.setUnit(clinicalfinding.getUnit());
            nodeInfo.setRange(clinicalfinding.getRange());
            opBillNeoDTO.getClinicfindings().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<Vital> vitals = operation.getVitals();
        for (Vital vital : vitals) {
            nodeInfo = new NodeInfo();
            String name = vital.getName();
            if (name.contains("禁忌")) {
                name = name.substring(name.indexOf("禁忌")+2);
            }
            nodeInfo.setName(name);
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(vital.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(vital.getMaxval()));
            nodeInfo.setUnit(vital.getUnit());
            nodeInfo.setRange(vital.getRange());
            opBillNeoDTO.getVitals().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<LIS> lisSet = operation.getConflict_lis();
        for (LIS lis : lisSet) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(lis.getName());
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(lis.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(lis.getMaxval()));
            nodeInfo.setUnit(lis.getUnit());
            nodeInfo.setRange(lis.getRange());
            opBillNeoDTO.getLis().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<OralMedicine> conflict_oralmeds = operation.getOralMedicines();
        for (OralMedicine conflict_oralmed : conflict_oralmeds) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(conflict_oralmed.getName());
            opBillNeoDTO.getOralmeds().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<Disease> conflict_disease = operation.getConflict_diseases();
        for (Disease disease : conflict_disease) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(disease.getName());
            opBillNeoDTO.getDisease().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<Group> conflict_group = operation.getGroups();
        for (Group group : conflict_group) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(group.getName());
            opBillNeoDTO.getGroup().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<PACSDescribe> conflict_pacsdesc = operation.getConflict_pacsdesc();
        for (PACSDescribe pacsdesc : conflict_pacsdesc) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(pacsdesc.getName());
            opBillNeoDTO.getPacsDesc().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        return opBillNeoDTO;
    }

    /**
     * 处理手术开单合理性
     */
    public List<BillNeoDTO> getOperationBill(Map<String, String> opterm, OperationRepository opRepository) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        OperationBillNeoDTO opBillNeoDTO;

        String opname;

        for (String term : opterm.keySet()) {
            billNeoDTO = new BillNeoDTO();
            opname = opterm.get(term);
            billNeoDTO.setName(opname);

            List<Operation> operations = opRepository.findByNameIs(opname);

            if (null!=operations && operations.size()>0) {
                opBillNeoDTO = OperationtoOperationDTO(operations.get(0));
                billNeoDTO.setOperationBillNeoDTO(opBillNeoDTO);
            }

            billNeoDTOs.add(billNeoDTO);
        }

        return billNeoDTOs;
    }

    /**
     * 获取手术操作名称列表
     */
    public List<OperationInfoDTO> getOperationList(String name, OperationRepository operationRepository) {
        String nodename;

        List<OperationInfoDTO> opslist = new ArrayList<>();
        OperationInfoDTO opInfoDTO;

        List<Operation> operations = operationRepository.findByNameLike("*"+name+"*");

        for (Operation op : operations) {
            opInfoDTO = new OperationInfoDTO();
            nodename = op.getName();

            opInfoDTO.setName(nodename);
            opslist.add(opInfoDTO);
        }


        return opslist;
    }



    /**
     * 获取高危手术和操作数据
     */
    public List<HighRiskNeoDTO> getHighRiskOperation(Map<String, String> opsterm, OperationRepository operationRepository) {
        List<HighRiskNeoDTO> highRiskNeoDTOS = new ArrayList<>();
        HighRiskNeoDTO highRiskNeoDTO;
        String opname;
        Operation operation;

        for (String term : opsterm.keySet()) {
            highRiskNeoDTO = new HighRiskNeoDTO();

            opname = opsterm.get(term);
            highRiskNeoDTO.setName(term);
            highRiskNeoDTO.setStandname(opname);

            List<Operation> operations = operationRepository.findByNameIs(opname);

            if (null != operations) {
                operation = operations.get(0);
                if (null!=operation.getRisklevel() && operation.getRisklevel().trim().length()>0) {
                    highRiskNeoDTO.setIsHighrisk(true);
                    highRiskNeoDTO.setType("手术");
                    highRiskNeoDTOS.add(highRiskNeoDTO);
                }
            }
        }

        return highRiskNeoDTOS;
    }

}
