package com.diagbot.repository;

import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.PacsBillNeoDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.entity.node.*;
import com.diagbot.util.NeoUtil;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class PACSNode {


    public PacsBillNeoDTO PacstoPACSDTO(PACS pacs) {
        PacsBillNeoDTO pacsBillNeoDTO = new PacsBillNeoDTO();
        NodeInfo nodeInfo;

        Age age = pacs.getAge();
        if (age!=null) {
            NodeNeoDTO ageNeoDTO = new NodeNeoDTO();
            ageNeoDTO.setMin(BigDecimal.valueOf(age.getMinval()));
            ageNeoDTO.setMax(BigDecimal.valueOf(age.getMaxval()));
            pacsBillNeoDTO.setAgeNeoDTO(ageNeoDTO);
        }


        Gender gender = pacs.getGender();
        if (gender!=null) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(gender.getName());
            pacsBillNeoDTO.setGender(NeoUtil.updateNodeInfo(nodeInfo));
        }



        Set<ClinicalFinding> clinicalfindings = pacs.getClinicalfindings();
        for (ClinicalFinding clinicalfinding : clinicalfindings) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(clinicalfinding.getName());
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(clinicalfinding.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(clinicalfinding.getMaxval()));
            nodeInfo.setUnit(clinicalfinding.getUnit());
            nodeInfo.setRange(clinicalfinding.getRange());
            pacsBillNeoDTO.getClinicfindings().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<Vital> vitals = pacs.getVitals();
        for (Vital vital : vitals) {
            nodeInfo = new NodeInfo();
            String name = vital.getName();
            if (name.contains("禁忌")) {
                name = name.substring(name.indexOf("禁忌")+2);
            }
            nodeInfo.setName(name);
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(vital.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(vital.getMaxval()));
            nodeInfo.setUnit(vital.getUnit());
            nodeInfo.setRange(vital.getRange());
            pacsBillNeoDTO.getVitals().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<OralMedicine> conflict_medicines = pacs.getOralmeds();
        for (OralMedicine conflict_medicine : conflict_medicines) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(conflict_medicine.getName());
            pacsBillNeoDTO.getOralmeds().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<AllergicMed> allergicmeds = pacs.getAllergicmeds();
        for (AllergicMed allergicmed : allergicmeds) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(allergicmed.getName());
            pacsBillNeoDTO.getAllergicmeds().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<Operation> operations = pacs.getOperations();
        for (Operation operation : operations) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(operation.getName());
            pacsBillNeoDTO.getOperations().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<Procedure> procedures = pacs.getProcedures();
        for (Procedure procedure : procedures) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(procedure.getName());
            pacsBillNeoDTO.getProcedures().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<LIS> liss = pacs.getLiss();
        for (LIS lis : liss) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(lis.getName());
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(lis.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(lis.getMaxval()));
            nodeInfo.setUnit(lis.getUnit());
            nodeInfo.setRange(lis.getRange());
            pacsBillNeoDTO.getLis().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<Disease> diseases = pacs.getDiseases();
        for (Disease disease : diseases) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(disease.getName());

            pacsBillNeoDTO.getDisease().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<Group> groups = pacs.getGroups();
        for (Group group : groups) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(group.getName());

            pacsBillNeoDTO.getGroup().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<PACSDescribe> conflict_pacsdesc = pacs.getPacs_conflict_pacsdesc();
        for (PACSDescribe pacsdesc : conflict_pacsdesc) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(pacsdesc.getName());

            pacsBillNeoDTO.getPacsDesc().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        return pacsBillNeoDTO;
    }


    /**
     * 处理辅检开单合理性
     */
    public List<BillNeoDTO> getPacsBill(Map<String, String> pacsterm, PacsRepository pacsRepository) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        PacsBillNeoDTO pacsBillNeoDTO;

        List<PACS> pacslt;
        String pacsname;

        for (String term : pacsterm.keySet()) {
            billNeoDTO = new BillNeoDTO();
            pacsname = pacsterm.get(term);
            billNeoDTO.setName(term);
            billNeoDTO.setStandname(pacsname);

            pacsBillNeoDTO = new PacsBillNeoDTO();

            pacslt = pacsRepository.findByNameIs(pacsname);

            if (pacslt != null && pacslt.size()>0) {
                pacsBillNeoDTO = PacstoPACSDTO(pacslt.get(0));
            }

            billNeoDTO.setPacsBillNeoDTO(pacsBillNeoDTO);

            billNeoDTOs.add(billNeoDTO);
        }

        return billNeoDTOs;
    }


    /**
     * 获取辅检名称列表
     */
    public List<String> getPACSNameList(String name, PacsRepository pacsRepository) {
        String pacsname;

        List<String> pacslist = new ArrayList<>();

        List<PACS> pacslt = pacsRepository.findByNameLike("*"+name+"*");

        for (PACS pacs : pacslt) {
            pacsname = pacs.getName();
            if (!pacslist.contains(pacsname)) {
                pacslist.add(pacsname);
            }
        }


        return pacslist;
    }
}






